@@ -461,7 +461,7 @@ impl Watcher {
     }
 
     /// Polls for a new event, with an optional timeout. If no `timeout`
-    /// is passed, then it will block forever.
+    /// is passed, then it will return immediately.
     pub fn poll(&self, timeout: Option<Duration>) -> Option<Event> {
         // poll will not block indefinitely
         // None -> return immediately
@@ -471,6 +471,16 @@ impl Watcher {
         }
     }
 
+    /// Polls for a new event, with an optional timeout. If no `timeout`
+    /// is passed, then it will block until an event is received.
+    pub fn poll_forever(&self, timeout: Option<Duration>) -> Option<Event> {
+        if timeout.is_some() {
+            self.poll(timeout)
+        } else {
+            get_event(self, None)
+        }
+    }
+
     /// Creates an iterator that iterates over the queue. This iterator will block
     /// until a new event is received.
     pub fn iter(&self) -> EventIter {
@@ -478,6 +488,12 @@ impl Watcher {
     }
 }
 
+impl AsRawFd for Watcher {
+    fn as_raw_fd(&self) -> RawFd {
+        self.queue
+    }
+}
+
 impl Drop for Watcher {
     fn drop(&mut self) {
         unsafe { libc::close(self.queue) };
@@ -654,6 +670,7 @@ mod tests {
     use super::{EventData, EventFilter, FilterFlag, Ident, Vnode, Watcher};
     use std::fs;
     use std::io::Write;
+    use std::os::unix::io::{AsRawFd, FromRawFd};
     use std::path::Path;
     use tempfile;
 
@@ -828,4 +845,41 @@ mod tests {
         assert!(watcher.iter().next().is_some());
         assert!(watcher.iter().next().is_some());
     }
+
+    #[test]
+    fn test_nested_kqueue() {
+        let mut watcher = match Watcher::new() {
+            Ok(wat) => wat,
+            Err(_) => panic!("new failed for main watcher"),
+        };
+        let mut nested_watcher = match Watcher::new() {
+            Ok(wat) => wat,
+            Err(_) => panic!("new failed for nested watcher"),
+        };
+
+        let kqueue_file = unsafe { fs::File::from_raw_fd(nested_watcher.as_raw_fd()) };
+        assert!(
+            watcher
+                .add_file(&kqueue_file, EventFilter::EVFILT_READ, FilterFlag::empty())
+                .is_ok(),
+            "add_file failed for main watcher"
+        );
+
+        let mut file = tempfile::tempfile().unwrap();
+        assert!(
+            nested_watcher
+                .add_file(&file, EventFilter::EVFILT_VNODE, FilterFlag::NOTE_WRITE,)
+                .is_ok(),
+            "add_file failed for nested watcher"
+        );
+
+        assert!(watcher.watch().is_ok(), "watch failed on main watcher");
+        assert!(
+            nested_watcher.watch().is_ok(),
+            "watch failed on nested watcher"
+        );
+        assert!(file.write_all(b"foo").is_ok(), "write failed");
+        assert!(watcher.iter().next().is_some());
+        assert!(nested_watcher.iter().next().is_some());
+    }
 }
